<?php
/**
 * @version		$Id: buzer.php 1.0 2013-03-24 16:00:00Z $
 * @package		Joomla.Plugin
 * @subpackage	Content
 * @copyright	Copyright (C) 2013 buzer.de All rights reserved.
 * @license		This program is free software: you can redistribute it and/or modify
 *				it under the terms of the GNU General Public License as published by
 *				the Free Software Foundation, either version 3 of the License, or
 *				(at your option) any later version.
 *
 *				This program is distributed in the hope that it will be useful,
 *				but WITHOUT ANY WARRANTY; without even the implied warranty of
 *				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *				GNU General Public License for more details.
 *
 *				You should have received a copy of the GNU General Public License
 *				along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// No direct access.
defined('_JEXEC') or die;

// Import Joomla! Plugin library file
jimport('joomla.plugin.plugin');
jimport('joomla.installer.adapters.plugin');

class plgContentBuzer extends JPlugin {
	private $buzer_cache = 'buzer_cache';
	private $done = 0;
	private $db = null; 
	private $buzer_param = "";
	private $buzer_config = array (
	
		'version' => '1.01',

		# beliebige Anzahl Attribute zur Linkformatierung des Titels 
		# z.B. 'at'=>'target="_blank"' erzeugt Links wie <a href="ziel" target="_blank">...
		# kann beispielsweise auch fr die Formatierung (style, class, etc.) 
		# oder auch zum Plazieren von Skripten verwendet werden
		'at' 	=> 'target="_blank"',
		
		# analog zu 'at' nur die verlinkten Paragraphen/Artikel betreffend
		'ap'	=> 'target="_blank"',

		#  Zitierrichtung, Standand ist  1 der ZPO, d.h. der Titel wird rechts 
		# vom Paragraphen erwartet, dann ist hier keine Angabe notwendig, 
		# wird z.B. ZPO  1, 2, 3 etc. verwendet, funktioniert das Verlinken 
		# nur wenn das Flag 'lr' gesetz ist, z.B. 'lr' => 1
		'lr'	=> 0,
		
		# falls true, werden die Titel selbst nicht verlinkt, also 
		# z.B. bei  1 BGB wird nur der  1 verlinkt, allerdings 
		#erhlt dann natrlich auch "die Vorschriften des BGB bestimmen" keinen Link
		'not'	=> 0,
		
		# kommaseparierte Liste von Titeln, die mit diesem Titel nie verlinkt werden
		# WEG meint hfiger den Weg als das WEG, ALG ist oft ALG II
		'notitle' => 'WEG,ALG,LAG,FANG,WHG,ARG,WO',
		
		# falls true, werden Sonderzeichen html-codiert
		'esc'	=> 0,
		
		# ersetzt aufeinanderfolgende Leerzeichen durch eines, 
		# fhrt zu besseren Ergebnissen, 
		# falls Text stark durch Verwendung von Leerzeichen formatiert ist, 
		# zerstrt dann allerdings auch diese Formatierung
		'trim'	=> 0,
		
		# Zieladresse fr Postheader
		'_post'		=> '/li.htm',

		# Servername
		'_host'	=> 'www.buzer.de',
		
		# Port
		'_port'		=> 80,
		
		# timeout fr Verbindung zu buzer.de in Sekunden, falls berschritten 
		# wird der Eingangstext zurckgegeben
		'_timeout'	=> 5,
		
		# false hier eine Angabe erfolgt, wird eine Tabelle dieses Namens
		# als Cache verwendet
		# bei Angabe erfolgt die Verlinkung bei Anzeige des Artikels
		# wenn leer, wird der Artikel beim Speichern in der Datenbank verlinkt
		'_cache'	=> 'buzer_cache',
		
		# sollte wenn die Tabelle angelegt wurde, auf 1 gesetzt werden,
		# sonst wird jedesmal erst versucht die Tabelle zu erstellen
		'_cache_exists' => 1,
		
		# wird bei phpBB nicht verwendet
		'charset'	=> 'utf-8', 
		
		# Debug falls true
		'_debug'	=> 0,
		
	);	

	function plgContentBuzer( &$subject, $params ) {
		parent::__construct( $subject, $params );
		
	}

	public function onContentPrepare($context, &$article, &$params, $page = 0) {
		global $mainframe;
		$this->db = JFactory::getDbo();
		
		$buzer_config=$this->buzer_config;
		foreach ($buzer_config as $key => $value) {
			if ($this->params->get($key)) {
				$this->buzer_config[$key] = $this->params->get($key);
			}
		}
		if ( $buzer_config['_debug'] ) {
			foreach ($this->buzer_config as $key => $value) {
				$article->text .= "$key => ".$this->buzer_config[$key]."<br />";
			}		
		}
		
		foreach ($this->buzer_config as $key => $value) {
			if ( $value && !preg_match('/^_/', $key )) {
				$this->buzer_param .= "&$key=" . urlencode( $value );
			}
		}		
		
		$article->text = $this->buzer_LinkIt($article->text);
		return true;
	}

	function buzer_getCache( $config, $text) {
		# gibt falls vorhandenen den cache zurck
		$sql = "select cache from ".$config['_cache']." 
			where hash = '".md5($this->buzer_param.$text)."' and charset='".mysql_escape_string($config['charset'])."'
			limit 1;";
		$this->db->setQuery($sql);
		$textout = $this->db->loadResult();

		# cache begrenzen
		if (time()%100==0) {
			$sql = "delete from ".$config['_cache']." where dateinsert<DATE_SUB(NOW(), INTERVAL 20 DAY);";
			$this->db->setQuery($sql);
			$this->db->query();			
		}
		
		return $textout=='1'?$text:$textout;
		
	}

	function buzer_insertCache( $config, $text, $cache) {
		# schreibt die verlinkte version in den cache
		$sql = "replace into ".$config['_cache']." (cache, hash, charset, dateinsert) 
			values (convert('".mysql_escape_string($text." "==$cache?1:$cache)."' using latin1), '".md5($this->buzer_param.$text)."', '".mysql_escape_string($config['charset'])."', now())";
		$this->db->setQuery($sql);
		$this->db->query();
		
	}

	function buzer_LinkIt($textin) {
		# verlinkt den bergebenen text
		$buzer_config=$this->buzer_config;
		
		if ( !$textin ) {
			return $textin;
		}
				
		if ( $cache=$this->buzer_getCache( $buzer_config, $textin) ) { return $cache; }	

		if ( $this->done ) {
			return $textin;
		}
		
		# maximal ein request pro Seitenabruf
		$this->done = 1;
		
		$content = 'plugin=joomla2.5&site='.urlencode($_SERVER['HTTP_HOST']); 
		
		// foreach ($buzer_config as $key => $value) {
			// if ( $value && !preg_match('/^_/', $key )) {
				// $content .= "&$key=" . urlencode( $value );
			// }
		// }
		$content .= $this->buzer_param;
		
		$content .= "&text=" . urlencode($textin);
		
		$redirect = 0;
		$attempt = 0;
		while ( !$attempt++ and $redirect<2 ) {
			$header = "POST ".$buzer_config['_post']." HTTP/1.0\n";
			$header .= "Host: ".$buzer_config['_host']."\n";
			$header .= "Content-type: application/x-www-form-urlencoded; charset=" . urlencode($buzer_config['charset']) ."\n";
			$header .= "Content-length: " . strlen($content) . "\n\n";
			
			if ( $buzer_config['_debug']) echo "header=$header<br />";
			
			$socket = fsockopen($buzer_config['_host'], $buzer_config['_port'], $errno, $errstr, $buzer_config['_timeout']);
			
			if ( $buzer_config['_debug']) echo "fsockopen: socket=$socket, errno=$errno, errstr=$errstr<br />";
			
			if ( !$socket ) return $textin;
			
			if ( $buzer_config['_debug']) echo "Content: $content<br />";

			socket_set_timeout($socket, $buzer_config['_timeout'], 0);
			socket_set_blocking($socket, true);
			fputs($socket, $header.$content);
			$timed_out = false;
			$eof = false;
			$textout='';
			while (!$eof and !$timed_out) {
				$textout.= fgets($socket, 1024);
				$state = socket_get_status($socket);
				$eof = $state["eof"];
				$timed_out = $state["timed_out"];
			}
			fclose($socket);
			if ( $buzer_config['_debug']) echo "length: ".strlen($textout)."<br />";
			
			$del = "\n";
			if ( preg_match("/\r\n/", $textout) ) { $del="\r\n"; }
			
			$ok = false;
			$lines = explode($del, $textout);
			$i = 0;
			while ( $_ = $lines[$i++] ) {
				if ( preg_match('/200 OK/', $_) ) {
					$ok = true;
				}
				if ( preg_match('/location:.*?\/\/([^\/]+)(\/.+)/i',$_,$loc) ) {
					$buzer_config['_host'] = $loc[1];
					$buzer_config['_post'] = $loc[2];
					$attempt = 0;
					$redirect++;
					break;
				}
			}
		}
		
		$textout = implode($del, array_slice( $lines, $i));
		
		if ( $buzer_config['_debug']) echo "read: ok=$ok, textout=$textout<br />";
		
		if ( !$ok ) {
			return $textin;
		}	
		
		$cmd = substr( $textout, -100);
		if ( substr($cmd, 0, 2) != "OK" ) {
			return $textin;
		}
		$textout = substr( $textout, 0, -100);
		

		if ( $textin and strlen($textout)<strlen($textin) ) {
			if ( $buzer_config['_debug']) echo "Break zu kurz<br />";
			return $textin;
		} 
		
		$this->buzer_insertCache( $buzer_config, $textin, $textout);
		return $textout;

	}	
	
	
	
}
